"""
    SleekXMPP: The Sleek XMPP Library
    Copyright (C) 2010 Nathanael C. Fritz
    This file is part of SleekXMPP.

    See the file LICENSE for copying permission.
"""
__all__ = ['xep_0004', 'xep_0009', 'xep_0012', 'xep_0030', 'xep_0033',
           'xep_0045', 'xep_0050', 'xep_0060', 'xep_0066', 'xep_0082',
           'xep_0085', 'xep_0086', 'xep_0092', 'xep_0128', 'xep_0199',
           'xep_0203', 'xep_0224', 'xep_0249', 'gmail_notify']

# Don't automatically load xep_0078
